# -*- coding: utf-8 -*-
from odoo import models, fields, api
from odoo import tools
from datetime import datetime

class LeFilamentTdb(models.Model):
	""" SAPOVAL DASHBOARD """
	_name = "lefilament.dashboard"
	_description = "Le Filament Dashboard"
	_order = "date_tdb desc"

	name = fields.Char( 'Mois', compute='get_month')
	date_tdb = fields.Date( 'Date', required=True, default=datetime.today().strftime('%Y-%m-%d'),)

	ca_mois = fields.Float( 'Facturé', compute="dashboard_values", store=True)

	cmd_mois = fields.Float( 'Commandes', compute="dashboard_values", store=True)	

	pipe_mois = fields.Float( 'Pipe', compute="dashboard_values", store=True )

	
	treso = fields.Float( 'Trésorerie', compute="dashboard_values", store=True )
	variation = fields.Float( 'Variation', compute="dashboard_values", store=True )
	charges = fields.Float( 'Décaissé', compute="dashboard_values", store=True )
	encaisse = fields.Float( 'Encaissé', compute="dashboard_values", store=True )
	charges_fixes = fields.Float( 'Charges Fixes', default=10000 )
	runway = fields.Float( 'Runway', compute="runway_value", )

	@api.multi
	@api.depends('date_tdb')
	def dashboard_values(self):
		for record in self:
			if record.date_tdb :
				date_tdb = datetime.strptime(record.date_tdb, '%Y-%m-%d')
				
				##############    CA    ################
				# FACTURÉ
				self.env.cr.execute("select sum(amount_untaxed) from account_invoice where state!='draft' and type='out_invoice' and date >= date_trunc('month', %s) and date < date_trunc('month', %s + interval '1' month);", (date_tdb, date_tdb) )
				ca_mois = self.env.cr.fetchone()[0]

				##############    COMMANDES    ################
				# TOTAL
				self.env.cr.execute("select sum(amount_untaxed) from sale_order where invoice_status='to invoice' and date_order >= date_trunc('month', %s) and date_order < date_trunc('month', %s + interval '1' month);", (date_tdb, date_tdb) )
				cmd_mois = self.env.cr.fetchone()[0]

				##############    TRESO   ################
				# Trésorerie
				self.env.cr.execute("select sum(amount) from account_bank_statement_line where date < date_trunc('month', %s + interval '1' month);", (date_tdb, ) )
				treso_total = self.env.cr.fetchone()[0]
				# CHARGES
				self.env.cr.execute("select sum(amount) from account_bank_statement_line where amount < 0 and date >= date_trunc('month', %s) and date < date_trunc('month', %s + interval '1' month);", (date_tdb, date_tdb) )
				charges = self.env.cr.fetchone()[0]
				# ENCAISSE
				self.env.cr.execute("select sum(amount) from account_bank_statement_line where amount > 0 and date >= date_trunc('month', %s) and date < date_trunc('month', %s + interval '1' month);", (date_tdb, date_tdb) )
				encaisse = self.env.cr.fetchone()[0]

				##############    CHARGES   ################
				self.env.cr.execute("select charges_fixes from res_company" )
				charges_fixes = self.env.cr.fetchone()[0]

				##############    PIPE    ################
				# TOTAL
				self.env.cr.execute("select sum(planned_revenue*probability/100) from crm_lead where active=True;")
				pipe = self.env.cr.fetchone()[0]

				if not encaisse:
					encaisse = 0
				if not charges:
					charges = 0

				record.ca_mois = ca_mois
				record.cmd_mois = cmd_mois
				record.treso = treso_total
				record.charges = charges * (-1.0)
				record.encaisse = encaisse
				record.variation = encaisse + charges
				record.charges_fixes = charges_fixes
				record.pipe_mois = pipe


	@api.multi
	@api.depends('charges_fixes','treso')
	def runway_value(self):
		for record in self:
			if record.charges_fixes :
				record.runway = record.treso / record.charges_fixes

	@api.one
	def get_month(self):
		months = ['Janv', 'Fév', 'Mars', 'Avr', 'Mai', 'Juin', 'Juil', 'Août', 'Sept', 'Oct', 'Nov', 'Dec']
		date_tdb = self.date_tdb
		month = int(date_tdb[5:7])
		year = date_tdb[2:4]
		self.name = months[month-1] + " " + year


	@api.model
	def new_data(self):
		self.create({ 'date_tdb': str(datetime.now()) })

	@api.model
	def retrieve_datas_dashboard(self):
		res = {
			'facture': 0,
			'commandes': 0,
			'pipe': 0,
			'pipe_win': 0,
			'pipe_to_win': 0,
			'pipe_n1': 0,
			'tresorerie': 0,
			'entree': 0,
			'sortie': 0,
			'variation': 0,
			'target': 0,
			'cca': 0,
			'capital': 0,
			'date_maj': 0,
			'a_encaisser': 0,
			'a_payer': 0,
		}

		self._cr.execute("""
			SELECT
            	(select count(*) from account_invoice) as id,
            	(select sum(amount_untaxed) from account_invoice where state!='draft' and type='out_invoice' and date >= date_trunc('year', current_date) and date < date_trunc('year', current_date + interval '1' year) ) as facture, 
            	(select sum(residual_company_signed) from account_invoice where state!='draft' and type='out_invoice' ) as a_encaisser, 
            	(select sum(residual_company_signed) from account_invoice where state!='draft' and type='in_invoice' ) as a_payer, 
            	(select sum(planned_revenue*probability/100) from crm_lead where active=True and (date_deadline < date_trunc('year', current_date  + interval '1' year) or date_deadline is null) ) as pipe, 
            	(select sum(planned_revenue*probability/100) from crm_lead where active=True and date_deadline >= date_trunc('year', current_date + interval '1' year) ) as pipe_n1, 
            	(select sum(planned_revenue*probability/100) from crm_lead where active=True and probability=100  and (date_deadline < date_trunc('year', current_date + interval '1' year) or date_deadline is null) ) as pipe_win, 
            	(select sum(planned_revenue*probability/100) from crm_lead where active=True and probability!=100  and (date_deadline < date_trunc('year', current_date + interval '1' year) or date_deadline is null) ) as pipe_to_win,
            	(select date from account_bank_statement ORDER BY ID DESC LIMIT 1) as date_maj,
            	(select sum(amount) from account_bank_statement_line ) as tresorerie,
            	(select sum(amount) from account_bank_statement_line where amount > 0 and date >= date_trunc('year', current_date) ) as entree,
            	(select sum(amount) from account_bank_statement_line where amount < 0 and date >= date_trunc('year', current_date) ) as sortie,
            	(select sum(amount) from account_bank_statement_line where date >= date_trunc('year', current_date) ) as variation,
            	(select sum(es.total_amount) from hr_expense_sheet es, hr_expense e where es.id = e.sheet_id and e.payment_mode='own_account' and es.state!='done' ) as cca,
            	(select sum(price_subtotal-qty_invoiced*price_unit) from sale_order_line where invoice_status='to invoice') as commandes; """)
		datas = self._cr.dictfetchall()

		self._cr.execute("select ca_target from res_company;")
		ca_target = self._cr.dictfetchall()

		self._cr.execute("select sum(capital) as capital from hr_employee;")
		capital = self._cr.dictfetchall()

		if datas[0]['facture']:
			res['facture'] =+ datas[0]['facture']
		if datas[0]['a_encaisser']:
			res['a_encaisser'] =+ datas[0]['a_encaisser']
		if datas[0]['a_payer']:
			res['a_payer'] =+ datas[0]['a_payer']
		if datas[0]['pipe']:
			res['pipe'] =+ datas[0]['pipe']
		if datas[0]['pipe_win']:
			res['pipe_win'] =+ datas[0]['pipe_win']
		if datas[0]['pipe_to_win']:
			res['pipe_to_win'] =+ datas[0]['pipe_to_win']
		if datas[0]['pipe_n1']:
			res['pipe_n1'] =+ datas[0]['pipe_n1']
		if datas[0]['tresorerie']:
			res['tresorerie'] =+ datas[0]['tresorerie']
		if datas[0]['date_maj']:
			res['date_maj'] = datas[0]['date_maj']
		if datas[0]['entree']:
			res['entree'] =+ datas[0]['entree']
		if datas[0]['sortie']:
			res['sortie'] =+ datas[0]['sortie']
		if datas[0]['variation']:
			res['variation'] =+ datas[0]['variation']
		if datas[0]['commandes']:
			res['commandes'] =+ datas[0]['commandes']
		if datas[0]['cca']:
			res['cca'] =+ datas[0]['cca']
		if ca_target[0]['ca_target']:
			res['target'] =+ ca_target[0]['ca_target']
		if capital[0]['capital']:	
			res['capital'] =+ capital[0]['capital']

		return res

	@api.model
	def tresorerie(self):
		self._cr.execute("""SELECT to_char(date_trunc('month', date),'YYYY-MM') as mois, 
					sum(case when amount > 0 then amount else 0 end ) as entree,
					sum(case when amount < 0 then amount else 0 end ) as sortie,
					sum(amount) as variation
					from account_bank_statement_line 
					group by date_trunc('month', date) 
					order by date_trunc('month', date);""")
		tresorerie = self._cr.dictfetchall()

		self._cr.execute("""SELECT 
					sum(es.total_amount) as cca
					from hr_expense_sheet es, hr_expense e 
					where es.id = e.sheet_id and e.payment_mode='own_account' and es.state!='done';""")
		cca = self._cr.dictfetchall()

		return { 'tresorerie': tresorerie, 'cca': cca }

